# GenomicAnnotations.jl

## Description
GenomicAnnotations is a package for reading, modifying, and writing genomic annotations in the GenBank format.

## Usage
GenBank files are read with `readgbk(gbkfile)`, which returns a vector of `Chromosome`s. If we're only interested in the first chromosome in `example.gbk` we only need to store the first element.
```julia
chr = readgbk("test/example.gbk")[1]
```

`Chromosome`s have five fields, `name`, `header`, `genes`, `genedata`, and `sequence`. The `name` is read from the `header`, which is stored as a string. The annotation data is stored in `genedata`, but generally you should use `genes` to access that data. For example, it can be used to iterate over annotations, and to modify them.
```julia
for gene in chr.genes
    gene.locus_tag = "$(chr.name)_$(gene.locus_tag)"
end

chr.genes[2].locus_tag = "test123"
```

Accessing properties that haven't been stored will return missing. For this reason, it often makes more sense to use `get()` than to access the property directly.
```julia
if chr.genes[2].pseudo
    println("Gene 2 is a pseudogene")
end
# chr.genes[2].pseudo returns missing, so this will throw an error

if get(chr.genes[2], :pseudo, false)
    println("Gene 2 is a pseudogene")
end
```

The macro `@genes` can be used to filter through the annotations. The keyword `gene` is used to refer to the individual `Gene`s. `@genes` can also be used to modify annotations.
```julia
@genes(chr, :feature == "CDS")
@genes(chr, length(gene) > 300)

# The following two expressions are equivalent:
@genes(chr, :feature == "CDS", length(gene) > 300)
@genes(chr, (:feature == "CDS") && (length(gene) > 300))

@genes(chr, :locus_tag == "tag03")[1].pseudo = true
delete!(@genes(chr, :pseudo)) # Delete all psudogenes
delete!(@genes(chr, length(gene) <= 60)) # Delete all genes 60 nt or shorter
```

Gene sequences can be accessed with `sequence(gene)`. For example, the following code will write the translated sequences of all protein-coding genes to a file:
```julia
using BioSequences
writer = FASTA.Writer(open("proteins.fasta", "w"))
for gene in @genes(chr, :feature == "CDS")
    protseq = translate(convert(RNASequence, sequence(gene)))
    write(writer, FASTA.record(gene.locus_tag, gene.product, protseq))
end
close(writer)
```

Genes can be added using `addgene!`, and `sort!` can be used to make sure that the resulting annotations are in the correct order.
```julia
newgene = addgene!(chr, "regulatory", 670:677)
newgene.locus_tag = "reg02"
sort!(chr.genes)
```

After modifying the annotations, `printgbk(io, chr)` can be used to write them to a file.
```julia
open("updated.gbk", "w") do f
    printgbk(f, chr)
end
```
